@@ -13,7 +13,7 @@ require Carp;
 use Mojo::Transaction;
 
 # Oh, so they have internet on computers now!
-our $VERSION = '0.8';
+our $VERSION = '0.8005';
 
 *build_tx = \&build_transaction;
 
@@ -35,15 +35,26 @@ Mojo - The Web In A Box!
     sub handler {
         my ($self, $tx) = @_;
 
-        # Do magic things!
+        # Hello world!
+        $tx->res->code(200);
+        $tx->res->headers->content_type('text/plain');
+        $tx->res->body('Congratulations, your Mojo is working!');
 
         return $tx;
     }
 
 =head1 DESCRIPTION
 
-L<Mojo> is a collection of libraries for web framework developers and example
-web frameworks.
+L<Mojo> is a collection of libraries and example web frameworks for web
+framework developers.
+
+If you are searching for a higher level MVC web framework you should take a
+look at L<Mojolicious>.
+
+Don't be scared by the amount of different modules in the distribution, they
+are all very loosely coupled.
+You can just pick the ones you like and ignore the rest, there is no
+tradeoff.
 
 For userfriendly documentation see L<Mojo::Manual>.
 
@@ -59,10 +70,27 @@ new ones.
     my $tx = $mojo->build_tx;
     my $tx = $mojo->build_transaction;
 
+Returns a new L<Mojo::Transaction> object;
+Meant to be overloaded in subclasses.
+
 =head2 C<handler>
 
     $tx = $mojo->handler($tx);
 
+Returns and takes a L<Mojo::Transaction> object as first argument.
+Meant to be overloaded in subclasses.
+
+    sub handler {
+        my ($self, $tx) = @_;
+
+        # Hello world!
+        $tx->res->code(200);
+        $tx->res->headers->content_type('text/plain');
+        $tx->res->body('Congratulations, your Mojo is working!');
+
+        return $tx;
+    }
+
 =head1 SUPPORT
 
 =head2 Web
@@ -95,6 +123,8 @@ Sebastian Riedel, C<sri@cpan.org>.
 
 In alphabetical order:
 
+Andreas Koenig.
+
 Andy Grundman
 
 Aristotle Pagaltzis
@@ -109,8 +139,12 @@ Jesse Vincent
 
 Marcus Ramberg
 
+Mark Stosberg
+
 Pedro Melo
 
+Robert Hicks
+
 Shu Cho
 
 vti
