@@ -14,30 +14,17 @@ require Scalar::Util;
 sub new {
     my $proto = shift;
 
-    # Create instance
-    my $class = ref $proto || $proto;
-    my $self = bless {}, $class;
-
-    # Shortcut
-    return $self unless @_;
-
     # Check attributes
     my $attrs;
     if (exists $_[1]) {
         my %attrs = (@_);
         $attrs = \%attrs;
     }
-    else { $attrs = $_[0] }
+    else { $attrs = $_[0] || {} }
 
-    # Attributes
-    for my $attr (keys %$attrs) {
-
-        # Attribute
-        if ($self->can($attr)) { $self->$attr($attrs->{$attr}) }
-
-        # No attribute, pass through to instance hash
-        else { $self->{$attr} = $attrs->{$attr} }
-    }
+    # Create instance
+    my $class = ref $proto || $proto;
+    my $self = bless $attrs, $class;
 
     return $self;
 }
@@ -60,12 +47,8 @@ sub attr {
     else { $options = $_[0] }
     $options ||= {};
 
-    Carp::croak('Option "filter" has to be a coderef')
-      if ($options->{filter} && ref $options->{filter} ne 'CODE');
-
     my $chained = delete $options->{chained};
     my $default = delete $options->{default};
-    my $filter  = delete $options->{filter};
     my $weak    = delete $options->{weak};
 
     undef $options;
@@ -84,6 +67,15 @@ sub attr {
         # Header
         my $code = "sub {\n";
 
+        # Warning gets optimized away
+        unless ($ENV{MOJO_BASE_OPTIMIZE}) {
+
+            # Check invocant
+            $code .= "${ws}Carp::croak(\'";
+            $code .= 'Attribute has to be called with an object not a class';
+            $code .= "')\n  ${ws}unless ref \$_[0];\n";
+        }
+
         # No value
         $code .= "${ws}if (\@_ == 1) {\n";
         unless (defined $default) {
@@ -106,24 +98,15 @@ sub attr {
         }
         $code .= "$ws}\n";
 
-        # Value
-        if ($filter) {
 
-            # Filter and store argument
-            $code .= "${ws}local \$_ = \$_[1];\n";
-            $code .= "$ws\$_[0]->{'$attr'} = \$filter->(\$_[0], \$_);\n";
+        # Store argument optimized
+        if (!$weak && !$chained) {
+            $code .= "${ws}return \$_[0]->{'$attr'} = \$_[1];\n";
         }
+
+        # Store argument the old way
         else {
-
-            # Store argument optimized
-            if (!$weak && !$chained) {
-                $code .= "${ws}return \$_[0]->{'$attr'} = \$_[1];\n";
-            }
-
-            # Store argument the old way
-            else {
-                $code .= "$ws\$_[0]->{'$attr'} = \$_[1];\n";
-            }
+            $code .= "$ws\$_[0]->{'$attr'} = \$_[1];\n";
         }
 
         # Weaken
@@ -166,10 +149,7 @@ Mojo::Base - Once Upon A Midnight Dreary!
     use base 'Mojo::Base';
 
     __PACKAGE__->attr('driver');
-    __PACKAGE__->attr('doors',
-        default => 2,
-        filter  => sub { s/\D//g; $_ }
-    );
+    __PACKAGE__->attr('doors', default => 2);
     __PACKAGE__->attr([qw/passengers seats/],
         chained => 1,
         default => sub { 2 }
@@ -190,15 +170,14 @@ Mojo::Base - Once Upon A Midnight Dreary!
 
 =head1 DESCRIPTION
 
-L<Mojo::Base> is a universal base class containing simple and fast helpers
-for object oriented Perl programming.
-
+L<Mojo::Base> is a base class containing simple and fast helpers for object
+oriented Perl programming.
 Main design goals are minimalism and staying out of your way.
-
-The syntax is a bit like L<Moose> or Ruby and the performance better than
+The syntax is a bit like Ruby and the performance better than
 L<Class::Accessor::Fast>.
-(Note that L<Mojo::Base> was never meant as a replacement for L<Moose>, both
-are solutions to completely different problems.)
+
+Note that this is just an accessor generator, take L<Moose> if you are
+looking for an object system.
 
 For debugging you can set the C<MOJO_BASE_DEBUG> environment variable.
 
@@ -222,7 +201,7 @@ For debugging you can set the C<MOJO_BASE_DEBUG> environment variable.
         default => 'foo'}
     );
 
-Currently there are four options supported.
+Currently there are three options supported.
 
     chained: Whenever you call an attribute with arguments the instance
              is returned instead of the value.
@@ -230,8 +209,6 @@ Currently there are four options supported.
              Note that the default value is "lazy", which means it only
              gets assigned to the instance after the attribute has been
              called.
-    filter:  Filters the value before assigning it to the instance,
-             must be a coderef.
     weak:    Weakens the attribute value.
 
 =cut
\ No newline at end of file
