@@ -22,7 +22,11 @@ __PACKAGE__->attr('buffer',
     chained => 1,
     default => sub { Mojo::Buffer->new }
 );
-__PACKAGE__->attr('build_start_line_callback', chained => 1);
+__PACKAGE__->attr([qw/
+    build_start_line_callback
+    prepare_builder_callback
+    prepare_parser_callback
+/], chained => 1);
 __PACKAGE__->attr('content',
     chained => 1,
     default => sub { Mojo::Content->new }
@@ -35,6 +39,8 @@ __PACKAGE__->attr([qw/major_version minor_version/],
 *to_string           = \&build;
 *body_params         = \&body_parameters;
 *build_start_line_cb = \&build_start_line_callback;
+*prepare_builder_cb  = \&prepare_builder_callback;
+*prepare_parser_cb   = \&prepare_parser_callback;
 
 # I'll keep it short and sweet. Family. Religion. Friendship.
 # These are the three demons you must slay if you wish to succeed in
@@ -61,6 +67,10 @@ sub body_length { shift->content->body_length }
 
 sub body_parameters {
     my $self = shift;
+
+    # Cached
+    return $self->{_body_parameters} if $self->{_body_parameters};
+
     my $params = Mojo::Parameters->new;
 
     # "x-application-urlencoded"
@@ -88,7 +98,8 @@ sub body_parameters {
         }
     }
 
-    return $params;
+    # Cache
+    return $self->{_body_parameters} = $params;
 }
 
 sub build {
@@ -147,13 +158,42 @@ sub build_start_line {
     return $startline;
 }
 
+sub cookie {
+    my ($self, $name) = @_;
+
+    # Shortcut
+    return undef unless $name;
+
+    # Map
+    unless ($self->{_cookies}) {
+        my $cookies = {};
+        for my $cookie (@{$self->cookies}) {
+            my $cname = $cookie->name;
+
+            # Multiple cookies with same name
+            if (exists $cookies->{$name}) {
+                $cookies->{$cname} = [$cookies->{$cname}]
+                  unless ref $cookies->{$cname} eq 'ARRAY';
+                push @{$cookies->{$cname}}, $cookie;
+            }
+
+            # Cookie
+            else { $cookies->{$cname} = $cookie }
+        }
+
+        # Cache
+        $self->{_cookies} = $cookies;
+    }
+
+    return $self->{_cookies}->{$name};
+}
+
 sub fix_headers {
     my $self = shift;
 
     # Content-Length header is required in HTTP 1.0 messages
-    my $length = $self->body_length;
-    if ($self->is_version('1.0') && !$self->is_chunked && $length) {
-        $self->headers->content_length($length)
+    if ($self->is_version('1.0') && !$self->is_chunked) {
+        $self->headers->content_length($self->body_length)
           unless $self->headers->content_length;
     }
 
@@ -177,6 +217,12 @@ sub get_header_chunk {
 sub get_start_line_chunk {
     my ($self, $offset) = @_;
 
+    # Prepare
+    if ($self->prepare_builder_cb && !$self->{_prepared_builder}) {
+        $self->prepare_builder_cb->($self);
+        $self->{_prepared_builder}++;
+    }
+
     # Start line generator
     return $self->build_start_line_cb->($self, $offset)
       if $self->build_start_line_cb;
@@ -211,6 +257,12 @@ sub is_version {
 sub parse {
     my $self = shift;
 
+    # Prepare
+    if ($self->prepare_parser_cb && !$self->{_prepared_parser}) {
+        $self->prepare_parser_cb->($self);
+        $self->{_prepared_parser}++;
+    }
+
     # Buffer
     $self->buffer->add_chunk(join '', @_) if @_;
 
@@ -223,18 +275,48 @@ sub parse {
     }
 
     # Done
-    $self->state('done') if $self->content->is_state('done');
+    $self->done if $self->content->is_done;
 
     return $self;
 }
 
 sub start_line_length { return length shift->build_start_line }
 
+sub upload {
+    my ($self, $name) = @_;
+
+    # Shortcut
+    return undef unless $name;
+
+    # Map
+    unless ($self->{_uploads}) {
+        my $uploads = {};
+        for my $upload (@{$self->uploads}) {
+            my $uname = $upload->name;
+
+            # Multiple uploads with same name
+            if (exists $uploads->{$name}) {
+                $uploads->{$uname} = [$uploads->{$uname}]
+                  unless ref $uploads->{$uname} eq 'ARRAY';
+                push @{$uploads->{$uname}}, $upload;
+            }
+
+            # Upload
+            else { $uploads->{$uname} = $upload }
+        }
+
+        # Cache
+        $self->{_uploads} = $uploads;
+    }
+
+    return $self->{_uploads}->{$name};
+}
+
 sub uploads {
     my $self = shift;
 
-    my $uploads = {};
-    return $uploads unless $self->is_multipart;
+    my @uploads;
+    return \@uploads unless $self->is_multipart;
 
     my $formdata = $self->_parse_formdata;
 
@@ -247,22 +329,15 @@ sub uploads {
         next unless $filename;
 
         my $upload = Mojo::Upload->new;
+        $upload->name($name);
         $upload->file($part->file);
         $upload->filename($filename);
         $upload->headers($part->headers);
 
-        # Multiple uploads with same name
-        if (exists $uploads->{$name}) {
-            $uploads->{$name} = [$uploads->{$name}]
-              unless ref $uploads->{$name} eq 'ARRAY';
-            push @{$uploads->{$name}}, $upload;
-        }
-
-        # Simple upload
-        else { $uploads->{$name} = $upload }
+        push @uploads, $upload;
     }
 
-    return $uploads;
+    return \@uploads;
 }
 
 sub version {
@@ -385,6 +460,28 @@ implements the following new ones.
     my $minor_version = $message->minor_version;
     $message          = $message->minor_version(1);
 
+=head2 C<prepare_builder_cb>
+
+=head2 C<prepare_builder_callback>
+
+    my $cb   = $message->prepare_builder_cb;
+    my $cb   = $message->prepare_builder_callback;
+    $message = $message->prepare_builder_callback(sub {
+        my $self = shift;
+        # Do stuff! :)
+    });
+
+=head2 C<prepare_parser_cb>
+
+=head2 C<prepare_parser_callback>
+
+    my $cb   = $message->prepare_parser_cb;
+    my $cb   = $message->prepare_parser_callback;
+    $message = $message->prepare_parser_callback(sub {
+        my $self = shift;
+        # Do stuff! :)
+    });
+
 =head2 C<raw_body_length>
 
     my $raw_body_length = $message->raw_body_length;
@@ -415,6 +512,8 @@ the following new ones.
     my $params = $message->body_params;
     my $params = $message->body_parameters;
 
+Returns a L<Mojo::Parameters> object, containing POST parameters.
+
 =head2 C<to_string>
 
 =head2 C<build>
@@ -433,6 +532,10 @@ the following new ones.
 
     my $string = $message->build_start_line;
 
+=head2 C<cookie>
+
+    my $cookie = $message->cookie('foo');
+
 =head2 C<fix_headers>
 
     $message = $message->fix_headers;
@@ -465,8 +568,16 @@ the following new ones.
 
     $message = $message->parse('HTTP/1.1 200 OK...');
 
+=head2 C<upload>
+
+    my $upload = $message->upload('foo');
+
+Returns a L<Mojo::Upload> object or a arrayref of L<Mojo::Upload> objects.
+
 =head2 C<uploads>
 
     my $uploads = $message->uploads;
 
+Returns a arrayref of L<Mojo::Upload> objects.
+
 =cut
\ No newline at end of file
