@@ -222,7 +222,7 @@ sub _parse_start_line {
             else {
                 $self->major_version(0);
                 $self->minor_version(9);
-                $self->state('done');
+                $self->done;
             }
         }
         else { $self->error('Parser error: Invalid request line') }
@@ -269,6 +269,9 @@ implements the following new ones.
     my $params = $req->params;
     my $params = $req->parameters;
 
+Returns a L<Mojo::Parameters> object, containing both GET and POST
+parameters.
+
 =head2 C<query_params>
 
 =head2 C<query_parameters>
@@ -276,6 +279,8 @@ implements the following new ones.
     my $params = $req->query_params;
     my $params = $req->query_parameters;
 
+Returns a L<Mojo::Parameters> object, containing GET parameters.
+
 =head2 C<url>
 
     my $url = $req->url;
