@@ -158,6 +158,7 @@ sub to_string {
         $value = Mojo::ByteStream->new($value)
           ->url_escape($Mojo::URL::PCHAR . '\/\?');
 
+        $value ||= '';
         push @params, "$name=$value";
     }
 
@@ -225,7 +226,7 @@ the following new ones.
 
 =head2 C<parameter>
 
-    my $foo = $params->params('foo');
+    my $foo = $params->param('foo');
     my $foo = $params->parameter('foo');
     my $foo = $params->parameter(foo => 'ba;r');
 
@@ -245,4 +246,4 @@ the following new ones.
 
     my $string = $params->to_string;
 
-=cut
\ No newline at end of file
+=cut
